<?php

namespace Pudo\WooCommerce;

use stdClass;
use WC_Order;

/**
 *  Copyright: © 2024 The Courier Guy
 */
class ApiRequestBuilder
{
    private WC_Order $order;
    private string $method;

    private array $settings;

    /**
     * @param WC_Order $order
     * @param string $method
     */
    public function __construct(WC_Order $order, string $method)
    {
        $this->order  = $order;
        $this->method = $method;
        $instanceId   = 0;
        foreach ($order->get_items('shipping') as $item_id => $item) {
            $instanceId = $item->get_instance_id();
        }

        $pudoShippingMethod = new Pudo_Shipping_Method($instanceId);
        $this->settings     = $pudoShippingMethod->instance_settings;
    }

    /**
     * @param string $serviceLevelCode
     *
     * @return stdClass
     */
    public function buildBookingRequest(string $serviceLevelCode): stdClass
    {
        $requestBody = new stdClass();

        $requestBody->delivery_contact   = $this->buildDeliveryContact();
        $requestBody->collection_contact = $this->buildCollectionContact();

        $requestBody->delivery_address   = $this->buildDeliveryAddress();
        $requestBody->collection_address = $this->buildCollectionAddress();

        $requestBody->service_level_code = $serviceLevelCode;

        return $requestBody;
    }

    /**
     * @return stdClass
     */
    private function buildDeliveryContact(): stdClass
    {
        $deliveryContact                = new stdClass();
        $deliveryContact->name          = $this->order->get_shipping_first_name();
        $deliveryContact->email         = $this->order->get_billing_email();
        $deliveryContact->mobile_number = $this->order->get_billing_phone();

        return $deliveryContact;
    }

    /**
     * @return stdClass
     */
    private function buildCollectionContact(): stdClass
    {
        $collectionContact                = new stdClass();
        $collectionContact->name          = $this->settings['sender_contact'];
        $collectionContact->email         = $this->settings['sender_email'];
        $collectionContact->mobile_number = $this->settings['sender_phone'];

        return $collectionContact;
    }

    /**
     * @param $order
     * @param $method
     * @param array $data
     *
     * @return stdClass
     */
    private function buildDeliveryAddress(): stdClass
    {
        $deliveryAddress = new stdClass();

        if ($this->method === 'L2L' || $this->method === 'D2L') {
            //Check if there is a destination locker name, if so extract only the code
            $pudoLockerDestination = $this->order->get_meta('pudo_locker_destination', true);
            if (str_contains($pudoLockerDestination, ':')) {
                $pudoLockerDestinationArr = explode(':', $pudoLockerDestination);
                $pudoLockerDestination    = $pudoLockerDestinationArr[0];
            }
            $deliveryAddress->terminal_id = $pudoLockerDestination;
        } else {
            $deliveryAddress->type            = 'residential';
            $deliveryAddress->street_address  = $this->order->get_shipping_address_1();
            $deliveryAddress->city            = $this->order->get_shipping_city();
            $deliveryAddress->code            = $this->order->get_shipping_postcode();
            $deliveryAddress->zone            = $this->order->get_shipping_state();
            $deliveryAddress->country         = $this->order->get_shipping_country();
            $deliveryAddress->entered_address = "$deliveryAddress->street_address, $deliveryAddress->city,
             $deliveryAddress->code";
        }

        return $deliveryAddress;
    }

    /**
     * @return stdClass
     */
    private function buildCollectionAddress(): stdClass
    {
        $collectionAddress = new stdClass();

        if ($this->method === 'L2L' || $this->method === 'L2D') {
            //Check if there is an origin locker name, if so extract only the code
            $pudoLockerOrigin = $this->order->get_meta('pudo_locker_origin', true);
            if (str_contains($pudoLockerOrigin, ':')) {
                $pudoLockerOriginArr = explode(':', $pudoLockerOrigin);
                $pudoLockerOrigin    = $pudoLockerOriginArr[0];
            }
            $collectionAddress->terminal_id = $pudoLockerOrigin;
        } else {
            $collectionAddress->type            = 'residential';
            $collectionAddress->street_address  = $this->settings['sender_addressline1'];
            $collectionAddress->local_area      = $this->settings['sender_suburb'];
            $collectionAddress->city            = $this->settings['sender_city'];
            $collectionAddress->code            = $this->settings['sender_postal_code'];
            $collectionAddress->entered_address = "$collectionAddress->street_address, $collectionAddress->city,
             $collectionAddress->code";
        }

        return $collectionAddress;
    }

    /**
     * @return stdClass
     */
    public function buildRatesRequest(): stdClass
    {
        $requestBody = new stdClass();

        $requestBody->collection_address = $this->buildCollectionAddress();
        $requestBody->delivery_address   = $this->buildDeliveryAddress();

        return $requestBody;
    }
}
