<?php

namespace Pudo\WooCommerce;

use Pudo\Common\Processor\APIProcessor;
use WP_Error;

/**
 *  Copyright: © 2024 The Courier Guy
 */
class PudoApi
{
    /**
     * @var array
     */
    public array $lockers;
    private string $accountKey;
    private string $apiURL;

    public function __construct()
    {
        $this->apiURL = (get_option('pudo_api_url')) ? get_option(
            'pudo_api_url'
        ) : 'https://sandbox.api-pudo.co.za';
        $this->accountKey = get_option('pudo_account_key') ?? '';
        try {
            $this->lockers = $this->getAllLockers();
        } catch (\Exception $exception) {
            die($exception->getMessage());
        }
    }

    /**
     * Make async booking request
     *
     * @param string $data
     *
     * @return array|Exception|string|WP_Error
     */
    public function bookingRequest(string $data): WP_Error|Exception|array|string
    {
        return $this->callPudoApi('booking_request', $data);
    }

    /**
     * @param string $data
     *
     * @return WP_Error|PromiseInterface|Exception|array|string|Response
     */
    public function getRates(string $data): WP_Error|PromiseInterface|Exception|array|string|Response
    {
        return $this->callPudoApi('get_rates', $data);
    }

    /**
     * @param int $bookingID
     *
     * @return array|Exception|string|WP_Error
     */
    public function getWaybill(int $bookingID): WP_Error|Exception|array|string
    {
        $data = "/$bookingID?api_key=$this->accountKey";

        return $this->callPudoApi('get_waybill', $data);
    }

    /**
     * @return array|Exception|string|WP_Error
     */
    public function getLockerRates(): WP_Error|Exception|array|string
    {
        return $this->callPudoApi('locker_rates');
    }

    /**
     * @return array
     */
    public function getAllLockers(): array
    {
        $mappedLockers = get_transient("pudo_Lockers_$this->apiURL");
        if (!$mappedLockers) {
            $response = $this->callPudoApi('get_all_lockers');
            if (is_array($response) && !empty($response['response']) && $response['response']['code'] === 200) {
                $mappedLockers = APIProcessor::mapLockers($response['body']);
                set_transient("pudo_Lockers_$this->apiURL", $mappedLockers, 24 * 60 * 60);
            } else {
                $mappedLockers = [];
            }
        }

        return $mappedLockers;
    }

    /**
     * @param $transactionId
     *
     * @return WP_Error|Exception|array|string
     */
    public function bookingConfirmation($transactionId): WP_Error|Exception|array|string
    {
        return $this->callPudoApi('bookingConfirmation', $transactionId);
    }

    /**
     * @param string $method
     * @param null $content
     *
     * @return WP_Error|Exception|array|string
     */
    private function callPudoApi(string $method, $content = null): WP_Error|Exception|array|string
    {
        $apiProcessor = new APIProcessor($this->apiURL);
        $request      = $apiProcessor->getRequest($method, $content);
        $url          = $request['url'];
        $type         = $request['type'];

        $response = '';

        try {
            if ($type === 'GET') {
                $response = wp_remote_get($url, [
                    'headers' => [
                        'Authorization' => "Bearer $this->accountKey"
                    ],
                    'timeout' => 30
                ]);
            } elseif ($type === 'POST') {
                $response = wp_remote_post($url, [
                    'headers'     => [
                        'Content-Type'  => 'application/json; charset=utf-8',
                        'Authorization' => "Bearer $this->accountKey"
                    ],
                    'body'        => $content,
                    'method'      => 'POST',
                    'data_format' => 'body',
                    'timeout'     => 30
                ]);
            }
        } catch (\Exception $exception) {
            return new Exception($exception->getMessage());
        }

        return $response;
    }
}
